// CPdfPageControl.cpp
//
// Copyright 1999-2000 Sander van der Wal
//
// $Id: pdfpagecontrol.cpp 1.1 2000-09-17 13:20:25+02 svdwal Exp svdwal $
//
// $Log: pdfpagecontrol.cpp $
// Revision 1.1  2000-09-17 13:20:25+02  svdwal
// Initial checkin
//

#include "PdfPageControl.h"

// CONE
#ifndef __COEMAIN_H__
#include <coemain.h>
#endif

// EIKON
#ifndef __EIKENV_H__
#include <eikenv.h>
#endif

// View
#include "FbsBitmapOutputDev.h"

// PDF
#include "Object.h"
#include "Catalog.h"
#include "Page.h"
#include "XRef.h"
#include "PDFDoc.h"
#include "FontEncoding.h"

#include "Pdf.hrh"
#include "Pdf.rsg"


// PROFILING
#include "PROFILE.h"


// Locals

static const TInt KBusyInitialDelay=200000; // in microseconds

static inline TInt round(double x) { return TInt(x+0.5); }

////////////////////////////////////////////////////////////////////////////
//
// class CPdfPageControl
//
////////////////////////////////////////////////////////////////////////////
CPdfPageControl::CPdfPageControl(PDFDoc* aDoc)
  :iDoc(aDoc)
{}


void CPdfPageControl::ConstructL()
{
  // create an empty bitmap
  iPageBitmap = new(ELeave) CWsBitmap(iCoeEnv->WsSession());
  User::LeaveIfError(iPageBitmap->Create(TSize(0,0), iCoeEnv->ScreenDevice()->DisplayMode()));

  // create the output device
  iOut = new(ELeave) CFbsBitmapOutputDev(this);
  iOut->ConstructL(iPageBitmap, iCoeEnv->ScreenDevice()->DisplayMode());

  // allocate resources for for multitasking rendering
  iDrawingMsg = iCoeEnv->AllocReadResourceL(R_DRAWING);
  iActivePage = CActivePage::NewL(0, this);
}
 

CPdfPageControl::~CPdfPageControl()
{
  delete iDrawingMsg;
  if (iPage) {
    iActivePage->Cancel();
    delete iActivePage;
  }
  delete iOut;
  delete iPageBitmap;
};


TSize CPdfPageControl::MinimumSize() const
{
  return TSize(0,0);
}


TSize CPdfPageControl::MaximumSize() const
{
  if (iPageBitmap)
    return iPageBitmap->SizeInPixels();
  else
    return TSize(0,0);
}


void CPdfPageControl::SetVisAreaOffset(TPoint aVisAreaOffset)
{
  iVisAreaOffset = aVisAreaOffset;
}


TSize CPdfPageControl::PageSizeInPixels()
{
  if (iPageBitmap)
    return iPageBitmap->SizeInPixels();
  else
    return TSize(0,0);
}


void CPdfPageControl::SetSelection(const TRect& aSelection)
{
  iSelection = aSelection;
}


void CPdfPageControl::Draw(const TRect& /*aRect*/) const
{
  if (iPageBitmap) {
    CWindowGc& gc=SystemGc();
  
    TSize bmVisSize(Min(iPageBitmap->SizeInPixels().iWidth,  Rect().Width() ),
                    Min(iPageBitmap->SizeInPixels().iHeight, Rect().Height()));
    TRect rect(iVisAreaOffset, bmVisSize);
    gc.BitBlt(iPosition, iPageBitmap, rect);
    if (!iSelection.IsEmpty()) {
      TRect sel = iSelection;
      sel.Move(iPosition.iX-iVisAreaOffset.iX, iPosition.iY-iVisAreaOffset.iY);
      // invert what's on the screen
      gc.SetDrawMode(CGraphicsContext::EDrawModeNOTSCREEN);
      gc.DrawRect(sel);
      gc.SetDrawMode(CGraphicsContext::EDrawModePEN);
    }
  }
}


void CPdfPageControl::NotifyDump()
{
  DrawNow();
}


void CPdfPageControl::CvtUserToDev(TReal aX, TReal aY, TPoint& aDev)
{
  iOut->cvtUserToDev(aX, aY, &aDev.iX, &aDev.iY);
  //aDev.iX = (int)(iCtm[0] * aX + iCtm[2] * aY + iCtm[4] + 0.5);
  //aDev.iY = (int)(iCtm[1] * aX + iCtm[3] * aY + iCtm[5] + 0.5);
}


void CPdfPageControl::CvtDevToUser(TPoint aDev, TReal& aX, TReal& aY)
{
  TReal x, y;
  iOut->cvtDevToUser(aDev.iX, aDev.iY, &x, &y);
  aX = x;
  aY = y;
  //aX = iICtm[0] * aDev.iX + iICtm[2] * aDev.iY + iICtm[4];
  //aY = iICtm[1] * aDev.iX + iICtm[3] * aDev.iY + iICtm[5];
}


void CPdfPageControl::StartRenderingL(TInt aPage, TInt aDpi, const TPdfPageControlMemento& aMemento)
{
  iActivePage->Cancel();

  // set bitmapsize in twips
  TSize sizeInTwips;
  if (aMemento.iRotate == 90 || aMemento.iRotate == 270) {
    sizeInTwips.iHeight= round(iDoc->getPageWidth(aPage) *KTwipsPerPoint);
    sizeInTwips.iWidth = round(iDoc->getPageHeight(aPage)*KTwipsPerPoint);
  }
  else {
    sizeInTwips.iWidth = round(iDoc->getPageWidth(aPage) *KTwipsPerPoint);
    sizeInTwips.iHeight= round(iDoc->getPageHeight(aPage)*KTwipsPerPoint);
  }
  
  iEikonEnv->BusyMsgL(*iDrawingMsg, EHRightVTop, KBusyInitialDelay);
 
  // start drawing the page
  iOut->InitPageRendering(sizeInTwips, aMemento);
  iActivePage->StartRenderingL(iOut, iDoc, aPage, aDpi, aMemento.iRotate);
  
  // if we get here, the page is being rendered, so clean up
  iPage = aPage;
  iDoLinks = aMemento.iDoLinks;
}


TBool CPdfPageControl::IsRendering()
{
  if (iActivePage)
    return iActivePage->IsActive();
  else
    return EFalse;
}
 

void CPdfPageControl::CancelRendering()
{
  if (iActivePage)
    iActivePage->Cancel();
  
  iEikonEnv->BusyMsgCancel();
}


void CPdfPageControl::DisplayLinks(TInt aPage)
{
  iDoc->DisplayLinks(iOut, aPage);
}


void CPdfPageControl::NotifyPageFinishedL(TInt aError)
{
  // draw the links
  if (aError == KErrNone && iDoLinks)
    DisplayLinks(iPage);

  iEikonEnv->BusyMsgCancel();
 
  User::LeaveIfError(aError);
}
